@@ -22,14 +22,23 @@ This module handles gradients and patterns.
 
 """
 
-import cairo
 from math import radians
-from copy import deepcopy
 
+from . import cairo
 from .colors import color
-from .helpers import node_format, preserve_ratio, urls, transform
+from .helpers import node_format, preserve_ratio, paint, urls, transform
+from .shapes import rect
 from .units import size
 from ..parser import Tree
+from ..features import match_features
+
+
+BLEND_OPERATORS = {
+    "normal": 2,
+    "multiply": 14,
+    "screen": 15,
+    "darken": 17,
+    "lighten": 18}
 
 
 def update_def_href(surface, def_name, def_dict):
@@ -37,15 +46,26 @@ def update_def_href(surface, def_name, def_dict):
     def_node = def_dict[def_name]
     href = def_node.get("{http://www.w3.org/1999/xlink}href")
     if href and href[0] == "#" and href[1:] in def_dict:
-        new_def_node = deepcopy(def_dict[href[1:]])
-        new_def_node.update(def_node)
-        def_node = new_def_node
-        def_dict[def_name] = def_node
+        href_urls = urls(href)
+        href_url = href_urls[0] if href_urls else None
+        href_name = href_url[1:]
+        if href_name in def_dict:
+            update_def_href(surface, href_name, def_dict)
+            href_node = def_dict[href_name]
+            def_dict[def_name] = Tree(
+                url="#%s" % def_name, parent=href_node,
+                parent_children=(not def_node.children),
+                tree_cache=surface.tree_cache)
+            # Inherit attributes generally not inherited
+            for key, value in href_node.items():
+                if key not in def_dict[def_name]:
+                    def_dict[def_name][key] = value
 
 
 def parse_def(surface, node):
     """Parse the SVG definitions."""
-    for def_type in ("marker", "gradient", "pattern", "path", "mask"):
+    for def_type in (
+            "marker", "gradient", "pattern", "path", "mask", "filter"):
         if def_type in node.tag.lower():
             getattr(surface, def_type + "s")[node["id"]] = node
 
@@ -57,7 +77,7 @@ def gradient_or_pattern(surface, node, name):
         return draw_gradient(surface, node, name)
     elif name in surface.patterns:
         update_def_href(surface, name, surface.patterns)
-        return draw_pattern(surface, name)
+        return draw_pattern(surface, node, name)
 
 
 def marker(surface, node):
@@ -70,6 +90,11 @@ def mask(surface, node):
     parse_def(surface, node)
 
 
+def filter_(surface, node):
+    """Store a filter definition."""
+    parse_def(surface, node)
+
+
 def linear_gradient(surface, node):
     """Store a linear gradient definition."""
     parse_def(surface, node)
@@ -99,23 +124,21 @@ def paint_mask(surface, node, name, opacity):
     if mask_node.get("maskUnits") == "userSpaceOnUse":
         width_ref, height_ref = "x", "y"
     else:
-        x = float(size(surface, node.get("x"), "x"))
-        y = float(size(surface, node.get("y"), "y"))
-        width = float(size(surface, node.get("width"), "x"))
-        height = float(size(surface, node.get("height"), "y"))
+        x = size(surface, node.get("x"), "x")
+        y = size(surface, node.get("y"), "y")
+        width = size(surface, node.get("width"), "x")
+        height = size(surface, node.get("height"), "y")
         width_ref = width
         height_ref = height
         mask_node["transform"] = "%s scale(%f, %f)" % (
             mask_node.get("transform", ""), width, height)
 
-    mask_node["x"] = float(
-        size(surface, mask_node.get("x", "-10%"), width_ref))
-    mask_node["y"] = float(
-        size(surface, mask_node.get("y", "-10%"), height_ref))
-    mask_node["height"] = float(
-        size(surface, mask_node.get("height", "120%"), width_ref))
-    mask_node["width"] = float(
-        size(surface, mask_node.get("width", "120%"), height_ref))
+    mask_node["x"] = size(surface, mask_node.get("x", "-10%"), width_ref)
+    mask_node["y"] = size(surface, mask_node.get("y", "-10%"), height_ref)
+    mask_node["height"] = size(
+        surface, mask_node.get("height", "120%"), width_ref)
+    mask_node["width"] = size(
+        surface, mask_node.get("width", "120%"), height_ref)
 
     if mask_node.get("maskUnits") == "userSpaceOnUse":
         x = mask_node["x"]
@@ -128,6 +151,9 @@ def paint_mask(surface, node, name, opacity):
     mask_surface = SVGSurface(mask_node, None, surface.dpi, surface)
     surface.context.save()
     surface.context.translate(x, y)
+    surface.context.scale(
+        mask_node["width"] / mask_surface.width,
+        mask_node["height"] / mask_surface.height)
     surface.context.mask_surface(mask_surface.cairo)
     surface.context.restore()
 
@@ -142,25 +168,25 @@ def draw_gradient(surface, node, name):
         width_ref, height_ref = "x", "y"
         diagonal_ref = "xy"
     else:
-        x = float(size(surface, node.get("x"), "x"))
-        y = float(size(surface, node.get("y"), "y"))
-        width = float(size(surface, node.get("width"), "x"))
-        height = float(size(surface, node.get("height"), "y"))
+        x = size(surface, node.get("x"), "x")
+        y = size(surface, node.get("y"), "y")
+        width = size(surface, node.get("width"), "x")
+        height = size(surface, node.get("height"), "y")
         width_ref = height_ref = diagonal_ref = 1
 
     if gradient_node.tag == "linearGradient":
-        x1 = float(size(surface, gradient_node.get("x1", "0%"), width_ref))
-        x2 = float(size(surface, gradient_node.get("x2", "100%"), width_ref))
-        y1 = float(size(surface, gradient_node.get("y1", "0%"), height_ref))
-        y2 = float(size(surface, gradient_node.get("y2", "0%"), height_ref))
+        x1 = size(surface, gradient_node.get("x1", "0%"), width_ref)
+        x2 = size(surface, gradient_node.get("x2", "100%"), width_ref)
+        y1 = size(surface, gradient_node.get("y1", "0%"), height_ref)
+        y2 = size(surface, gradient_node.get("y2", "0%"), height_ref)
         gradient_pattern = cairo.LinearGradient(x1, y1, x2, y2)
 
     elif gradient_node.tag == "radialGradient":
-        r = float(size(surface, gradient_node.get("r", "50%"), diagonal_ref))
-        cx = float(size(surface, gradient_node.get("cx", "50%"), width_ref))
-        cy = float(size(surface, gradient_node.get("cy", "50%"), height_ref))
-        fx = float(size(surface, gradient_node.get("fx", str(cx)), width_ref))
-        fy = float(size(surface, gradient_node.get("fy", str(cy)), height_ref))
+        r = size(surface, gradient_node.get("r", "50%"), diagonal_ref)
+        cx = size(surface, gradient_node.get("cx", "50%"), width_ref)
+        cy = size(surface, gradient_node.get("cy", "50%"), height_ref)
+        fx = size(surface, gradient_node.get("fx", str(cx)), width_ref)
+        fy = size(surface, gradient_node.get("fy", str(cy)), height_ref)
         gradient_pattern = cairo.RadialGradient(fx, fy, 0, cx, cy, r)
 
     if gradient_node.get("gradientUnits") != "userSpaceOnUse":
@@ -181,21 +207,52 @@ def draw_gradient(surface, node, name):
         cairo, "EXTEND_%s" % gradient_node.get("spreadMethod", "pad").upper()))
 
     surface.context.set_source(gradient_pattern)
+    return True
 
 
-def draw_pattern(surface, name):
+def draw_pattern(surface, node, name):
     """Draw a pattern image."""
     pattern_node = surface.patterns[name]
     pattern_node.tag = "g"
-    transform(surface, "translate(%s %s)" % (
-        pattern_node.get("x"), pattern_node.get("y")))
     transform(surface, pattern_node.get("patternTransform"))
 
+    if pattern_node.get("viewBox"):
+        if not (size(surface, pattern_node.get("width", 1), 1) and
+                size(surface, pattern_node.get("height", 1), 1)):
+            return False
+    else:
+        if not (size(surface, pattern_node.get("width", 0), 1) and
+                size(surface, pattern_node.get("height", 0), 1)):
+            return False
+
+    if pattern_node.get("patternUnits") == "userSpaceOnUse":
+        x = size(surface, pattern_node.get("x"), "x")
+        y = size(surface, pattern_node.get("y"), "y")
+        pattern_width = size(surface, pattern_node.get("width", 0), 1)
+        pattern_height = size(surface, pattern_node.get("height", 0), 1)
+    else:
+        width = size(surface, node.get("width"), "x")
+        height = size(surface, node.get("height"), "y")
+        x = size(surface, pattern_node.get("x"), 1) * width
+        y = size(surface, pattern_node.get("y"), 1) * height
+        pattern_width = \
+            size(surface, pattern_node.pop("width", "0"), 1) * width
+        pattern_height = \
+            size(surface, pattern_node.pop("height", "0"), 1) * height
+        if "viewBox" not in pattern_node:
+            pattern_node["width"] = pattern_width
+            pattern_node["height"] = pattern_height
+            if pattern_node.get("patternContentUnits") == "objectBoundingBox":
+                pattern_node["transform"] = "scale(%s, %s)" % (width, height)
     from . import SVGSurface  # circular import
     pattern_surface = SVGSurface(pattern_node, None, surface.dpi, surface)
     pattern_pattern = cairo.SurfacePattern(pattern_surface.cairo)
     pattern_pattern.set_extend(cairo.EXTEND_REPEAT)
+    pattern_pattern.set_matrix(cairo.Matrix(
+        pattern_surface.width / pattern_width, 0, 0,
+        pattern_surface.height / pattern_height, -x, -y))
     surface.context.set_source(pattern_pattern)
+    return True
 
 
 def draw_marker(surface, node, position="mid"):
@@ -253,8 +310,8 @@ def draw_marker(surface, node, position="mid"):
 
                 width, height, viewbox = node_format(surface, marker_node)
                 if viewbox:
-                    viewbox_width = viewbox[2] - viewbox[0]
-                    viewbox_height = viewbox[3] - viewbox[1]
+                    viewbox_width = viewbox[2]
+                    viewbox_height = viewbox[3]
                 else:
                     viewbox_width = width or 0
                     viewbox_height = height or 0
@@ -276,6 +333,65 @@ def draw_marker(surface, node, position="mid"):
         node.pending_markers.append(pending_marker)
 
 
+def apply_filter_before(surface, node):
+    if node["id"] in surface.masks:
+        return
+
+    names = urls(node.get("filter"))
+    name = names[0][1:] if names else None
+    if name in surface.filters:
+        filter_node = surface.filters[name]
+        for child in filter_node.children:
+            # Offset
+            if child.tag == "feOffset":
+                if filter_node.get("primitiveUnits") == "objectBoundingBox":
+                    width = size(surface, node.get("width"), "x")
+                    height = size(surface, node.get("height"), "y")
+                    dx = size(surface, child.get("dx", 0), 1) * width
+                    dy = size(surface, child.get("dy", 0), 1) * height
+                else:
+                    dx = size(surface, child.get("dx", 0), 1)
+                    dy = size(surface, child.get("dy", 0), 1)
+                surface.context.translate(dx, dy)
+
+
+def apply_filter_after(surface, node):
+    surface.context.set_operator(BLEND_OPERATORS["normal"])
+
+    if node["id"] in surface.masks:
+        return
+
+    names = urls(node.get("filter"))
+    name = names[0][1:] if names else None
+    if name in surface.filters:
+        filter_node = surface.filters[name]
+        for child in filter_node.children:
+            # Blend
+            if child.tag == "feBlend":
+                surface.context.set_operator(BLEND_OPERATORS.get(
+                    child.get("mode", "normal"), BLEND_OPERATORS["normal"]))
+            # Flood
+            elif child.tag == "feFlood":
+                surface.context.new_path()
+                if filter_node.get("primitiveUnits") == "objectBoundingBox":
+                    x = size(surface, node.get("x"), "x")
+                    y = size(surface, node.get("y"), "y")
+                    width = size(surface, node.get("width"), "x")
+                    height = size(surface, node.get("height"), "y")
+                else:
+                    x, y, width, height = 0, 0, 1, 1
+                x += size(surface, child.get("x", 0), 1)
+                y += size(surface, child.get("y", 0), 1)
+                width *= size(surface, child.get("width", 0), 1)
+                height *= size(surface, child.get("height", 0), 1)
+                rect(surface, dict(x=x, y=y, width=width, height=height))
+                surface.context.set_source_rgba(*color(
+                    paint(child.get("flood-color"))[1],
+                    float(child.get("flood-opacity", 1))))
+                surface.context.fill()
+                surface.context.new_path()
+
+
 def use(surface, node):
     """Draw the content of another SVG file."""
     surface.context.save()
@@ -290,10 +406,24 @@ def use(surface, node):
     if "mask" in node:
         del node["mask"]
     href = node.get("{http://www.w3.org/1999/xlink}href")
-    url = list(urls(href))[0]
-    tree = Tree(url=url, parent=node)
+    tree_urls = urls(href)
+    url = tree_urls[0] if tree_urls else None
+    tree = Tree(url=url, parent=node, tree_cache=surface.tree_cache)
+
+    if not match_features(tree.xml_tree):
+        return
+
+    if tree.tag == "svg":
+        # Explicitely specified
+        # http://www.w3.org/TR/SVG11/struct.html#UseElement
+        if "width" in node and "height" in node:
+            tree["width"], tree["height"] = node["width"], node["height"]
+
     surface.set_context_size(*node_format(surface, tree))
     surface.draw(tree)
+    node.pop("fill", None)
+    node.pop("stroke", None)
     surface.context.restore()
     # Restore twice, because draw does not restore at the end of svg tags
-    surface.context.restore()
+    if tree.tag != "use":
+        surface.context.restore()
