@@ -20,9 +20,9 @@ Surface helpers.
 
 """
 
-import cairo
 from math import cos, sin, tan, atan2, radians
 
+from . import cairo
 from .units import size
 
 # Python 2/3 management
@@ -43,17 +43,25 @@ def distance(x1, y1, x2, y2):
     return ((x2 - x1) ** 2 + (y2 - y1) ** 2) ** 0.5
 
 
-def filter_fill_or_stroke(value):
-    """Remove unnecessary characters from fill or stroke value."""
-    if not value:
-        return
+def paint(value):
+    """Extract from value an uri and a color.
 
-    content = list(urls(value))[0]
-    if "url" in value:
-        if not content.startswith("#"):
-            return
-        content = content[1:]
-    return content
+    See http://www.w3.org/TR/SVG/painting.html#SpecifyingPaint
+
+    """
+    if not value:
+        return None, None
+
+    value = value.strip()
+
+    if value.startswith("url"):
+        source = urls(value.split(")")[0])[0][1:]
+        color = value.split(")", 1)[-1].strip() or None
+    else:
+        source = None
+        color = value.strip() or None
+
+    return (source, color)
 
 
 def node_format(surface, node):
@@ -77,6 +85,7 @@ def normalize(string=None):
         string = string.replace("  ", " ")
 
     string = string.replace("e -", "e-")
+    string = string.replace("E -", "E-")
 
     values = string.split(" ")
     string = ""
@@ -234,8 +243,61 @@ def apply_matrix_transform(surface, matrix):
 
 def urls(string):
     """Parse a comma-separated list of url() strings."""
-    for link in string.split(","):
-        link = link.strip()
-        if link.startswith("url"):
-            link = link[3:]
-        yield link.strip("() ")
+    if not string:
+        return []
+
+    string = string.strip()
+    if string.startswith("url"):
+        string = string[3:]
+    return [
+        link.strip("() ") for link in string.rsplit(")")[0].split(",")
+        if link.strip("() ")]
+
+
+def rect(string):
+    """Parse the rect value of a clip."""
+    if not string:
+        return []
+    string = string.strip()
+    if string.startswith("rect"):
+        return string[4:].strip('() ').split(',')
+    else:
+        return []
+
+
+def rotations(node):
+    """Retrieves the original rotations of a `text` or `tspan` node."""
+    if "rotate" in node:
+        original_rotate = [
+            float(i) for i in normalize(node["rotate"]).strip().split(" ")]
+        return original_rotate
+    return []
+
+
+def pop_rotation(node, original_rotate, rotate):
+    """Removes the rotations of a node that are already used."""
+    node["rotate"] = " ".join(
+        str(rotate.pop(0) if rotate else original_rotate[-1])
+        for i in range(len(node.text)))
+
+
+def zip_letters(xl, yl, dxl, dyl, rl, word):
+    """Returns a list with the current letter's positions (x, y and rotation).
+    E.g.: for letter 'L' with positions x = 10, y = 20 and rotation = 30:
+    >>> [[10, 20, 30], 'L']
+
+    Store the last value of each position and pop the first one in order to
+    avoid setting an x,y or rotation value that have already been used.
+    """
+    return (
+        ([pl.pop(0) if pl else None for pl in (xl, yl, dxl, dyl, rl)], char)
+        for char in word)
+
+
+def flatten(node):
+    flattened_text = [node.text or ""]
+    for child in list(node):
+        flattened_text.append(flatten(child))
+        flattened_text.append(child.tail or "")
+        node.remove(child)
+    return "".join(flattened_text)
